--- chan_capi.c.orig	Fri Aug 13 12:07:28 2004
+++ chan_capi.c	Tue Mar 29 14:13:28 2005
@@ -32,12 +32,15 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/types.h>
-#include <linux/capi.h>
+#include <capi_bsd.h>
 #include <capi20.h>
+#include <capiutils.h>
 #include <asterisk/dsp.h>
 #include "xlaw.h"
 #include "chan_capi_pvt.h"
 
+typedef short unsigned int __u16;
+
 unsigned ast_capi_ApplID;
 _cword ast_capi_MessageNumber=1;
 static char *desc = "Common ISDN API for Asterisk";
@@ -81,7 +84,7 @@
 #else
 static int capi_capability = AST_FORMAT_ALAW;
 #endif
-static struct ast_capi_profile profile;
+static CAPIProfileBuffer_t profile;
 
 static pthread_t monitor_thread = -1;
 
@@ -154,9 +157,6 @@
     return 0;
 }
 
-unsigned Listen(unsigned long CIPmask) {
-    return ListenOnController(CIPmask,0);
-}
 
 // Echo cancellation is for cards w/ integrated echo cancellation only
 // (i.e. Eicon active cards support it)
@@ -628,7 +628,9 @@
     if (!len || len < (unsigned int) strip) return NULL;
     len = len - strip;
     data = (char *)(data + 1 + strip);
-    return strndup((char *)data,len);
+    char *x = strdup((char *)data);
+    x[len] = 0;
+    return x;
 }
 
 int capi_call(struct ast_channel *c, char *idest, int timeout)
@@ -640,6 +642,7 @@
 	char buffer[AST_MAX_EXTENSION];
 	char called[AST_MAX_EXTENSION],calling[AST_MAX_EXTENSION];
 	char bchaninfo[3];
+    long flags;
 	
 	_cmsg CMSG;
 	MESSAGE_EXCHANGE_ERROR  error;
@@ -684,10 +687,14 @@
 	if (pipe(fds) == 0) {
 	    ast_mutex_lock(&pipelock);
 	    i->fd = fds[0];
+        flags = fcntl(i->fd,F_GETFL);
+        fcntl(i->fd,F_SETFL,flags | O_SYNC | O_DIRECT);
 	    p = malloc(sizeof(struct capi_pipe));
 	    memset(p, 0, sizeof(struct capi_pipe));
 	    p->fd = fds[1];
-	    c->fds[0] = fds[1];
+        flags = fcntl(i->fd,F_GETFL);
+        fcntl(p->fd,F_SETFL,flags | O_SYNC | O_DIRECT);
+	    c->fds[0] = fds[0];
 	    p->PLCI = -1;
 	    p->i = i;
 	    p->c = c;
@@ -2642,41 +2649,36 @@
 	    return -1;
 	}
 
-	if (Listen(ALL_SERVICES) != 0) {
-	    ast_log(LOG_NOTICE,"unable to listen!\n");
-	    return -1;
-	}
-
-	if (capi20_get_profile(0,(char *)&profile) != 0) {
+	if (capi20_get_profile(0,&profile) != 0) {
 	    ast_log(LOG_NOTICE,"unable to get CAPI profile!\n");
 	    return -1;
 	} else {
-	    capi_num_controllers = profile.ncontrollers;
+	    capi_num_controllers = profile.wCtlr;
 	    if (option_verbose > 3)
 	    ast_verbose(VERBOSE_PREFIX_3 "This box has %d capi controller(s).\n",capi_num_controllers);
 	    for (controller=1;controller<=capi_num_controllers;controller++) {
 
 		memset(&profile,0,sizeof(profile));
-		capi20_get_profile(controller,(char *)&profile);
+		capi20_get_profile(controller,&profile);
 		cp = malloc(sizeof(struct ast_capi_controller));
 		cp->controller = controller;
-		cp->nbchannels = profile.nbchannels;
-		cp->nfreebchannels = profile.nbchannels;
-		if ((profile.globaloptions & 8) >> 3 == 1) {
+		cp->nbchannels = profile.wNumBChannels;
+		cp->nfreebchannels = profile.wNumBChannels;
+		if (profile.dwGlobalOptions & CAPI_PROFILE_DTMF_SUPPORT) {
 		    if (option_verbose > 3)
 			ast_verbose(VERBOSE_PREFIX_3 "CAPI[contr%d] supports DTMF\n",controller);
 		    cp->dtmf = 1;
 		} else {
 		    cp->dtmf = 0;	    
 		}
-		if (profile.globaloptions2 & 1) {
+		if (profile.dwGlobalOptions & CAPI_PROFILE_ECHO_CANCELLATION) {
 		    if (option_verbose > 3)
 			ast_verbose(VERBOSE_PREFIX_3 "CAPI[contr%d] supports echo cancellation\n",controller);
 		    cp->echocancel = 1;
 		} else {
 		    cp->echocancel = 0;
 		}
-		if ((profile.globaloptions & 16) >> 4 == 1) {
+		if (profile.dwGlobalOptions & CAPI_PROFILE_SUPPLEMENTARY_SERVICES)  {
 		    cp->sservices = 1;
 		} else {
 		    cp->sservices = 0;	    
